%%% -*- coding: latin-1 -*-

%%% The  contents of this  file are  subject to  the Erlang  Public License,
%%% Version  1.1, (the  "License");  you may  not  use this  file except  in
%%% compliance  with the License.  You should  have received  a copy  of the
%%% Erlang  Public License  along  with this  software.  If not,  it can  be
%%% retrieved at http://plc.inf.elte.hu/erlang/
%%%
%%% Software  distributed under  the License  is distributed  on an  "AS IS"
%%% basis, WITHOUT  WARRANTY OF ANY  KIND, either expressed or  implied. See
%%% the License  for the specific language governing  rights and limitations
%%% under the License.
%%%
%%% The Original Code is RefactorErl.
%%%
%%% The Initial Developer of the  Original Code is Etvs Lornd University.
%%% Portions created  by Etvs  Lornd University are  Copyright 2009,
%%% Etvs Lornd University. All Rights Reserved.

%%% @author Viktoria Fordos <v@fviktoria.hu>

%% -*- mode: nitrogen -*-

-module (files).
-compile(export_all).
-include_lib("nitrogen/include/wf.hrl").
-include("records.hrl").
-vsn("$Rev: 6460 $ ").

%%% ============================================================================
%%% HTML output

main() -> 
	case wf:role(auth_users) of
		true -> nitrogen_helper:init_qtab(),
				%nitrogen framework don't handle flashes container correctly, 
				%so unwanted flash messages may be displayed
				wf:session(flashes, undefined),
				#template { file=nitrogen_helper:get_nitrogen_site_dir()++
									 "/templates/files.html" };
		false ->wf:redirect_to_login("/index")
	end.

title() -> "RefactorErl Queries".

logout()->
	#link {text="Log out "++wf:user(), 
			title = "Log out "++wf:user(),
			postback=logout }.

get_server_root_dir()->
	TL=nitrogen_helper:get_file_browser_server_root(),
	if 
		length(TL) == 1 -> TL;
		true -> "__multi_dirs__"
	end.

get_db_root_dir()->
	TL=nitrogen_helper:get_file_browser_loaded_files_root(),
	if 
		length(TL) == 1 -> TL;
		true -> "__multi_dirs__"
	end.

show_file_section()->
	#panel{id=show_sorce_panel, body=""}.

browser_type_select()->
	Options=case wf:session(browser_type_dropdown) of
				undefined ->[#option{ text="Browse server", 
									  value="server", 
									  selected=true },
							 #option{ text="Browse loaded files", 
									  value="db", 
									  selected=false }];
				"server"  ->wf:session(browser_type_dropdown,undefined),
							[#option{ text="Browse server", 
									  value="server", 
									  selected=true },
							 #option{ text="Browse loaded files", 
									  value="db", 
									  selected=false }];
				"db"      ->wf:session(browser_type_dropdown,undefined),
							[#option{ text="Browse server", 
									  value="server", 
									  selected=false },
							 #option{ text="Browse loaded files", 
									  value="db", 
									  selected=true }]
			end,
	InnerCell=[#panel{id=buttons_placeholder,
					  body=[#link{id=show_file_button, 
								  body=[#image{image="/images/show_source.png", 
											   alt="Show source", 
											   class=no_decoration}], 
								  postback=show_file, 
								  class=no_decoration},
							#link{id=add_file_button, 
								  body=[#image{image="/images/database_add.png", 
											   alt="Add file", 
											   class=no_decoration}], 
								  postback={add_file,add},
								  class=no_decoration, 
								  actions=get_progress_event(
											"Adding file(s) to database ..")},
							#link{id=sync_file_button, 
								  body=[#image{image="/images/reload.png", 
											   alt="Reload", 
											   class=no_decoration}], 
								  postback={add_file,sync}, 
								  class=no_decoration,
								  actions=get_progress_event(
											"Reloading file(s) to database ..")},
							#link{id=delete_file_button, 
								  body=[#image{image=
												  "/images/database_remove.png", 
											   alt="Drop file", 
											   class=no_decoration}], 
								  postback=delete_file, class=no_decoration,
								  actions=get_progress_event(
											"Dropping file(s) from database ..")}]}],
	Body=#table{rows=[#tablerow{
							cells=[#tablecell{body=[
												#dropdown{id=browser_type_dropdown,
														  options=Options,
														  html_encode=true,
														  postback=load_file_browser}]},
								   #tablecell{body=InnerCell}]}]},
	Body.

message_panel()->
	StatusRow = #tablerow{cells=[#tablecell{id=spinner_progress,
											class=[spinner], 
											body=#image { image="/nitrogen/spinner.gif" }, 
											style="padding-right:5px;"},
								 #tablecell{body=#panel { class=flash_content, 
														  id=wait_message_panel, 
														  body="" }}]},
	
	InnerPanel = #panel { id=wait_message_flash_inner, 
						  class=flash, 
						  actions=#show { target=wait_message_flash, 
										  effect=blind, 
										  speed=10 }, 
						  body=[#link { class=flash_close_button, 
										text="Close", 
										actions=#event { type=click, 
														 target=wait_message_flash, 
														 actions=
															 #hide{effect=blind,
																   speed=400 }}},
								#table{rows=[StatusRow]}]},
    WaitPanel=#panel { id=wait_message_flash, 
					   style="display:none;", 
					   body=InnerPanel},
	wf:wire(wait_message_flash_inner, #hide{}),
	wf:wire(wait_message_flash, #hide{}),
	
	case wf:session(flash_message) of
		undefined -> ok;
		Message   -> wf:session(flash_message,undefined),
					 wf:flash(Message)
	end,
	[WaitPanel,#panel{id=message_panel,body=[#flash{}]}].

file_browser()->	
	Body=[#panel{id=file_browser_placeholder,
		   body=[#panel{id=file_browser_db_panel,
						body="<div id='file_browser_db'></div>"},
				 #panel{id=file_browser_server_panel,
						body="<div id='file_browser_server'></div>"}]}],
	wf:wire(file_browser_db_panel, #hide {}),
	wf:wire(sync_file_button, #hide {}),
	wf:wire(delete_file_button, #hide {}),
	Body.

%%% ============================================================================
%%% Handlers for postback events	

event(delete_file)->
	File=wf:session(selected_file),
	case File of
		undefined -> wf:wire(#alert{text="Please, select a file first!"});
		_ ->wf:wire(wait_message_flash, #hide{effect=blind, speed=400}),
			case nitrogen_helper:drop_from_db(File) of
				 {result, _}-> need_to_update(
								 "Successfully dropped "++
									 File++
									 " from database.");
				 {error, Error} ->wf:flash(Error)
			 end
	end;

event({add_file,Event})->
	File=wf:session(selected_file),
	case File of
		undefined -> wf:wire(#alert{text="Please, select a file first!"});
		_ ->wf:wire(wait_message_flash, #hide{effect=blind, speed=400}),
			case nitrogen_helper:add_to_db(File) of
				 {result, _}->	
					 case Event of
						 add -> need_to_update("Successfully added "++
												   File++
												   " to database.");
						 sync ->wf:flash("Successfully reloaded "++
											 File++
											 " to database.")
					 end;
				{error, Error} ->wf:flash(Error)
			 end
	end;

event(show_file)->
	File=wf:session(selected_file),
	case File of
		undefined -> wf:wire(#alert{text="Please, select file first!"});
		_ ->case nitrogen_helper:get_source(File) of
			{data, Data} ->
				wf:replace(show_sorce_panel,
						   #panel{id=show_sorce_panel,
								  body=[#p{body=File, class="query_title"},
										#textarea{id=source_code_textarea,
												  text=Data,
												  html_encode=true,
												  class="source_code_ta"}]}),
				wf:wire(#resize_textarea { targetId=source_code_textarea, 
										   maxWidth=80});
			{error, Error} ->
				wf:replace(show_sorce_panel,#panel{id=show_sorce_panel,
												   body=[#p{id=error_in_load,
															body=Error,
															class="error"}]})
			end
	end;
	

event(load_file_browser)->
	case wf:q(browser_type_dropdown) of
		"server"->wf:wire(file_browser_db_panel, #hide {}),
				  wf:wire(file_browser_server_panel, #show {}),
				  wf:wire(sync_file_button, #hide {}),
			  	  wf:wire(delete_file_button, #hide {}),
			  	  wf:wire(add_file_button, #show {}),
			  	  wf:wire(show_file_button, #show {});
		"db"->wf:wire(file_browser_server_panel, #hide {}),
			  wf:wire(file_browser_db_panel, #show {}),
			  wf:wire(sync_file_button, #show {}),
			  wf:wire(delete_file_button, #show {}),
			  wf:wire(add_file_button, #hide {}),
			  wf:wire(show_file_button, #show {})
	end;


event(logout) ->
	nitrogen_helper:delete_dependency_graphs_files(wf:user()),
	wf:logout(),
	nitrogen_helper:close_qtab(),
	wf:redirect("/index").

%%% ============================================================================
%%% Hepler functions
	
need_to_update(Message)->
	case nitrogen_helper:need_to_update() of
		false -> ok;
		true -> wf:session(flash_message,Message),
				wf:session(browser_type_dropdown,wf:q(browser_type_dropdown)),
				wf:redirect("/files")
	end.

get_progress_event(Label)->
	Event = #event { target=wait_message_panel, type=click },
	Actions=[#show{target=wait_message_flash_inner, speed=0},
			 #show{target=spinner_progress, speed=0},
			 #show {target=wait_message_flash, effect=blind, speed=10 }],
	Event#event{actions=[#set{value=Label}]++Actions}.
