%%% -*- coding: latin-1 -*-

%%% The  contents of this  file are  subject to  the Erlang  Public License,
%%% Version  1.1, (the  "License");  you may  not  use this  file except  in
%%% compliance  with the License.  You should  have received  a copy  of the
%%% Erlang  Public License  along  with this  software.  If not,  it can  be
%%% retrieved at http://plc.inf.elte.hu/erlang/
%%%
%%% Software  distributed under  the License  is distributed  on an  "AS IS"
%%% basis, WITHOUT  WARRANTY OF ANY  KIND, either expressed or  implied. See
%%% the License  for the specific language governing  rights and limitations
%%% under the License.
%%%
%%% The Original Code is RefactorErl.
%%%
%%% The Initial Developer of the  Original Code is Etvs Lornd University.
%%% Portions created  by Etvs  Lornd University are  Copyright 2009,
%%% Etvs Lornd University. All Rights Reserved.

%%% @author Viktoria Fordos <v@fviktoria.hu>

%% -*- mode: nitrogen -*-

-module (graphs).
-compile(export_all).
-include_lib("nitrogen/include/wf.hrl").
-include("records.hrl").
-vsn("$Rev: 6688 $ ").

%%% ============================================================================
%%% HTML output

main() -> 
	case wf:role(auth_users) of
		true ->
			nitrogen_helper:init_qtab(),
			#template { file=nitrogen_helper:get_nitrogen_site_dir()++
								 "/templates/graphs.html" };
		false ->wf:redirect_to_login("/index")
	end.

title() -> "RefactorErl Queries".

logout()->
	#link {text="Log out "++wf:user(), 
		   title = "Log out "++wf:user(), 
		   postback=logout}.

graph_control_panel()->
	LevelCell=#tablecell{body=[#label{text="Level:"},
							   #dropdown{id=graph_level,
										 options=[#option{ text="Function", 
														   value=func, 
														   selected=false},
												  #option{ text="Module", 
														   value=mod, 
														   selected=true }],
										 html_encode=true}],
						 class="graph_controll_tablecell"},
	TypeCell=#tablecell{body=[#label{text="Type:"},
							  #dropdown{id=graph_type,
										options=[#option{text="None", 
														 value=none, 
														 selected=true},
												 #option{ text="Whole graph", 
														  value=all, 
														  selected=false},
												 #option{ text="Cyclic sub- graph", 
														  value=cycles, 
														  selected=false}],
										html_encode=true,
										postback=type_changed}],
						class="graph_controll_tablecell"},
	SNodeModCell=#tablecell{body=[#label{text="Starting node (module)"},
								  #textbox_autocomplete {id=starting_node_module, 
														 tag=mods }],
							class="graph_controll_tablecell"},
	SNodeFunCell=#tablecell{body=[#label{text="Starting node (function)"},
								  #textbox_autocomplete {id=starting_node_function, 
														 tag=funs }],
							class="graph_controll_tablecell"},
	BtnCell=#tablecell{body=[#button{id=draw_graph,
									 text="Generate graph",
									 postback=generate_graph,
									 class=referl_button}],
					   colspan=2,
					   align="center"},
	ControlTable=#table{id=graph_control_panel,
						class="graph_controll_table",
						rows=[#tablerow{cells=[LevelCell,TypeCell]},
							  #tablerow{id=gnode,
										cells=[SNodeModCell,SNodeFunCell]},
							  #tablerow{cells=[BtnCell]}]},
	Form=#table{rows=[#tablerow{cells=[#tablecell{body=[ControlTable]},
									   #tablecell{body=[#panel{id=graph_result, 
															   body=[]}], 
												  valign="top", 
												  class="graph_controll_tablecell"}]}]},
	%see custom_validator's definition
	wf:wire(draw_graph, graph_type, 
			#validate { validators=[#custom { text="Please, select type or starting node", 
											  tag=some_tag, 
											  function=fun custom_validator/2}]}),
	Form.

%%% ============================================================================
%%% autocomplete

autocomplete_enter_event(SearchTerm, mods) ->
	Mods=case nitrogen_helper:execute_system_query("mods", 
												   {query_display_opt,
													[],
													needed_pattern,none}) of
		{result, no_result} -> [];
		{result, ResultMods} -> ResultMods
	end,
	DataM=lists:map(fun(Elem)-> 
						   {struct, [{id, list_to_binary(Elem)}, 
									 {label, list_to_binary(Elem)} , 
									 {value, list_to_binary(filename:rootname(
															  Elem)) }]} 
					end,Mods),
	List = [{struct,[{id, Id }, {label, Label}, {value, Value}]} || 
			{struct,[{id, Id }, {label, Label}, {value, Value}]} <- DataM, 
			string:str(string:to_lower(binary_to_list(Label)), 
					   string:to_lower(SearchTerm)) > 0],
	mochijson2:encode(List);

autocomplete_enter_event(SearchTerm, funs) ->
	Funs=case nitrogen_helper:execute_system_query("mods.funs", 
												   {query_display_opt,
													[],
													needed_pattern,$:}) of
		{result, no_result} -> [];
		{result, ResultFuns} -> ResultFuns
	end,	
	DataF=lists:map(fun(Elem)-> 
						   {struct, [{id, list_to_binary(Elem)}, 
									 {label, list_to_binary(Elem)} , 
									 {value, list_to_binary(Elem) }]} 
					end,Funs),
    List = [{struct,[{id, Id }, {label, Label}, {value, Value}]} || 
			{struct,[{id, Id }, {label, Label}, {value, Value}]} <- DataF, 
			string:str(string:to_lower(binary_to_list(Label)), 
					   string:to_lower(SearchTerm)) > 0],
	mochijson2:encode(List).

autocomplete_select_event({struct,[{<<"id">>,_},{<<"value">>,Value}]},_Tag)->
	wf:flash(Value),
	ok.

%%% ============================================================================
%%% validators

%Only postback the form, when at least one of those three is selected.
custom_validator(_Tag, _Value) ->
	case {wf:q(graph_type),
		  wf:q(starting_node_module), 
		  wf:q(starting_node_function)} of
		{"none", undefined, undefined} -> false;
		_ ->true
	end.

%%% ============================================================================
%%% Handlers for postback events	
event(type_changed)->
	case wf:q(graph_type) of
		"none" -> wf:wire(gnode,#show{});
		_      -> wf:wire(gnode,#hide{})
	end;

event(generate_graph)->
	Level=wf:q(graph_level),
	Type=wf:q(graph_type),
	GnodeM=wf:q(starting_node_module),
	GnodeF=wf:q(starting_node_function),
	Gnode=case Level of
			  "func"->funs;
			  "mod"-> mods;
			  _ ->    none
		  end,
				
	Result=case Type of
		"none" -> case Gnode of
					none -> nitrogen_helper:generate_dependency_graph(
							  [{level,list_to_atom(Level)}],wf:user());
					mods -> nitrogen_helper:generate_dependency_graph(
							  [{level,list_to_atom(Level)},
							   {gnode,list_to_atom(filename:basename(GnodeM,
																	 ".erl"))}],
							  wf:user());
					funs -> nitrogen_helper:generate_dependency_graph(
							  [{level,list_to_atom(Level)},
							   {gnode,GnodeF}],
							  wf:user())
				end;
		_ -> nitrogen_helper:generate_dependency_graph(
			   [{level,list_to_atom(Level)},
				{type, list_to_atom(Type)}],
			   wf:user())
	end,
	ResHtml=case Result of
		{{error, E1}, {error, E2}} when E1 == E2 -> [#p{body="Error: "++
																 wf:to_list(E1), 
														class="error"}];
		{{error, E1}, {error, E2}} when E1 /= E2 -> [#p{body="Errors: "++
																 wf:to_list(E1)++
																 " "++
																 wf:to_list(E2), 
														class="error"}];
		{DotName, {error, E1}}-> [#link{text="Generated graph in .dot", 
										title = "Download result", 
										url="images?image="++DotName },
								  #br{},
								  #br{},
								  #p{body="There was an error during svg generation: "++
											  wf:to_list(E1), 
									 class="error"}];
		{DotName, SvgName}-> [#link{text="Generated graph in .dot", 
									title = "Download result", 
									url="images?image="++DotName,
									style="margin:15px; padding:5px;" },
							  #br{},
							  #br{},
							  #link{text="See generated graph in .svg", 
									title = "See result in new window", 
									url="javascript:newWindow('graph_images?image="
								        ++SvgName++"');", 
									style="margin:15px; padding:5px;" }]
	end,
	wf:replace(graph_result,
			   #panel{id=graph_result, 
					  body=ResHtml, 
					  style="background-color:#FFF"}),
	wf:wire(graph_result,#hide{}),
	wf:wire(graph_result,#appear{});

event(logout) ->
	nitrogen_helper:delete_dependency_graphs_files(wf:user()),
	wf:logout(),
	nitrogen_helper:close_qtab(),
	wf:redirect("/index").
