%%% -*- coding: latin-1 -*-

%%% The  contents of this  file are  subject to  the Erlang  Public License,
%%% Version  1.1, (the  "License");  you may  not  use this  file except  in
%%% compliance  with the License.  You should  have received  a copy  of the
%%% Erlang  Public License  along  with this  software.  If not,  it can  be
%%% retrieved at http://plc.inf.elte.hu/erlang/
%%%
%%% Software  distributed under  the License  is distributed  on an  "AS IS"
%%% basis, WITHOUT  WARRANTY OF ANY  KIND, either expressed or  implied. See
%%% the License  for the specific language governing  rights and limitations
%%% under the License.
%%%
%%% The Original Code is RefactorErl.
%%%
%%% The Initial Developer of the  Original Code is Etvs Lornd University.
%%% Portions created  by Etvs Lornd University are Copyright 2009,
%%% Etvs Lornd University. All Rights Reserved.

%%% @author Viktoria Fordos <v@fviktoria.hu>

%% -*- mode: nitrogen -*-

-module (main).
-compile(export_all).
-include_lib("nitrogen/include/wf.hrl").
-include("records.hrl").
-vsn("$Rev: 6599 $ ").

-define(MAX_CHR_OF_LINE,25).

%%% ============================================================================
%%% HTML output

main() -> 
	case wf:role(auth_users) of
		true ->nitrogen_helper:init_qtab(),
			   #template { file=nitrogen_helper:get_nitrogen_site_dir()
						 ++"/templates/main.html" };
		false ->wf:redirect_to_login("/index")
	end.

title() -> "RefactorErl Queries".

query_editor()->
	Form=[
		 #textarea_autocomplete{ id=query_box, 
								 tag="", 
								 text="", 
								 minLength=2, 
								 delay=300, 
								 html_encode=true, 
								 next=new_query_button},
		#button{id=new_query_button, 
				text="Run", 
				postback=run_new_query, 
				class=[referl_button, run_new_query_button]}],
	wf:wire(new_query_button, 
			query_box, 
			#validate { validators=[#is_required { text="Required." }]}),
	Form.

query_result()->
	#panel{id=query_result_placeholder,body=[]}.

query_result_source_code()->
	#panel{id=query_result_source_code_placeholder,body=[]}.

executed_query()->
	#panel{id=executed_query_placeholder,body=[]}.

warnings()->
	Warning=nitrogen_helper:get_possible_warning(),
	case Warning of
		no_warning -> "";
		_ ->#panel{id=warning_messages_placeholder,body=Warning}
	end.

previous_queries()->
	Form=[#panel{id=placeholder,
				 body=[
						 #dropdown{id=query_owner,
								   options=[
											#option{ text="My queries", 
													 value="my", 
													 selected=true },
											#option{ text="All queries", 
													 value="all", 
													 selected=false }
											],
								   html_encode=true,
								   postback=load_list
								  },
						 #panel{id=queries_list_panel,
								body=[
									  #table{id=query_list,
											 class="query_list_table", 
											 rows=[]}
									  ]
								}
						 ]}],
	List=nitrogen_helper:query_list_from_qtab(wf:user()),
	LoadListFun=load_list_function(),
	lists:foreach(LoadListFun, List),
	Form.

running_queries()->
	wf:comet(fun() -> background_update() end),
	#panel{id=running_queries_placeholder,body=[]}.
	
logout()->
	#link {text="Log out "++wf:user(), 
		   title = "Log out "++wf:user(),
		   postback=logout }.

%%% ============================================================================
%%% autocomplete

autocomplete_enter_event(SearchTerm, _Tag) ->
	PosQueries=nitrogen_helper:do_autocomplete(SearchTerm),
	Data=lists:map(fun({Label,Value})-> 
						  {struct, [{id, list_to_binary(Value)}, 
									{label, list_to_binary(Label)} , 
									{value, list_to_binary(Value) }]} 
				   end,PosQueries),
	mochijson2:encode(Data).

autocomplete_select_event({struct,[{<<"id">>,_},{<<"value">>, Value}]},_Tag) ->
	wf:flash(Value),
	ok.

%%% ============================================================================
%%% Handlers for postback events	
run_query(Arg) ->
	nitrogen_helper:execute_query(Arg).

event(logout) ->
	nitrogen_helper:delete_dependency_graphs_files(wf:user()),
	wf:logout(),
	nitrogen_helper:close_qtab(),
	wf:redirect("/index");

event(run_new_query)->
	Q=wf:q(query_box),
	%selected file as starting point
	F=wf:session(selected_source),
	%selected position as starting point
	P=wf:session(selected_position),
	Pos=case {is_integer(P), is_list(P), is_atom(P)} of
		{true, false, false} -> P;
		{false, true, false} -> list_to_integer(P);
		{false, false, true} when P/=undefined->
			list_to_integer(atom_to_list(P))+1;
		{false, false, true} ->undefined
	end,							
	wf:replace(query_result_source_code_placeholder,
			   #panel{id=query_result_source_code_placeholder,body=[]}),
	wf:replace(executed_query_placeholder,
			   #panel{id=executed_query_placeholder,
					  body=[#p{body=nitrogen_helper:whitespaces_to_html(Q),
							   class="query_title"},
							#br{},
							#p{body="Running query, please wait.."},
							#br{}]}),
	wf:replace(query_result_placeholder, 
			   #panel{id=query_result_placeholder,body=[]}),
	{File, Position}=case string:chr(Q,$@) of
						 0 -> {undefined, undefined};
						 _ when (F/=undefined) and (Pos/=undefined) -> {F,Pos};
						 _ -> {undefined, undefined}
					 end,
	User=wf:user(),
	Fun = fun() -> run_query({'query',{Q, File, Position},User}) end,
    wf:continue({continue, Q}, Fun, 60*60*1000);
	%query_result_handler(Q,nitrogen_helper:execute_query({'query',
	%													  {Q, File, Position},
	%													  wf:user()})),
	%event(load_list);

event({run_query,SafeQuery,Q})-> 
	wf:replace(query_box,
			   #textarea_autocomplete{ id=query_box, 
									   tag="", 
									   text=Q, 
									   minLength=2, 
									   delay=300, 
									   html_encode=true, 
									   next=new_query_button}),
	wf:replace(query_result_source_code_placeholder,
			   #panel{id=query_result_source_code_placeholder,body=[]}),
	wf:replace(executed_query_placeholder,
			   #panel{id=executed_query_placeholder,
					  body=[#p{body=nitrogen_helper:whitespaces_to_html(Q),
							   class="query_title"},
							#br{},
							#p{body="Running query, please wait.."},
							#br{}]}),
	wf:replace(query_result_placeholder, 
			   #panel{id=query_result_placeholder,body=[]}),
	User=wf:user(),
	Fun = fun() -> run_query({prev_query,SafeQuery,User}) end,
    wf:continue({continue, Q}, Fun, 60*60*1000);
	%query_result_handler(Q,
	%					 nitrogen_helper:execute_query({prev_query,
	%													SafeQuery,
	%													wf:user()})),
	%event(load_list);

event({delete_query,SafeQuery})->
	nitrogen_helper:delete_from_qtab(SafeQuery,wf:user()),
	event(load_list);

event({abort_query,QueryId})->
	Message=case nitrogen_helper:kill_query(QueryId) of
		ok -> #p{body="Query aborted succesfully."};
		not_found -> #p{body="Query isn't found, sorry."}
	end,
	wf:replace(running_queries_placeholder,
			   #panel{id=running_queries_placeholder,
					  body=[Message]});

event({show_selection,File, StartPos, EndPos})->
	case nitrogen_helper:get_source(File) of
		{data, Data} ->
			wf:session(selected_source, File),
			wf:session(selected_position, StartPos-1),
			wf:replace(query_result_source_code_placeholder,
					   #panel{id=query_result_source_code_placeholder,
							  body=[#textarea{id=source_code_textarea,
											  text=Data,
											  html_encode=false,
											  class="source_code_ta"}]}),
			wf:wire(#resize_textarea { targetId=source_code_textarea, 
									   maxWidth=80}),
			wf:wire(#highlight_source { targetId=source_code_textarea, 
										startPos=StartPos, 
										endPos=EndPos}),
			wf:wire(#script{script="if (/Firefox/.test(navigator.userAgent))"++
						   "{obj('source_code_textarea').readOnly=true;}"});
		{error, Error} ->
			wf:replace(query_result_placeholder,
					   #panel{id=query_result_source_code_placeholder,
							  body=[#p{id=error_in_load,
									   body=Error,
									   class="error"}]})
	end;

event(load_list)->
	Who=wf:q(query_owner),
	wf:replace(query_list,
			   #table{id=query_list,class="query_list_table", rows=[]}),
	LoadListFun=load_list_function(),
	case Who of
		"my"->List=nitrogen_helper:query_list_from_qtab(wf:user()),
			  lists:foreach(LoadListFun, List);
		"all"->List=nitrogen_helper:query_list_from_qtab("all"),
			  lists:foreach(LoadListFun, List)
	end;

event({show_help,TempIdLink,TempIdCell,TempIdRow})->
	HelpLink=#link {id=TempIdLink,
												   text="?",
													class="delete_query_a", 
													title = "Hide information", 
													postback={hide_help,
															  TempIdLink, 
															  TempIdCell,
															  TempIdRow}},
	wf:replace(TempIdLink,HelpLink),
	wf:wire(TempIdRow, #appear{speed=50});

event({hide_help,TempIdLink,TempIdCell,TempIdRow})->
	HelpLink=#link {id=TempIdLink,
												   text="?",
													class="delete_query_a", 
													title = "Show information", 
													postback={show_help,
															  TempIdLink, 
															  TempIdCell,
															  TempIdRow}},
	wf:replace(TempIdLink,HelpLink),
	wf:wire(TempIdRow, #fade{speed=50});

event({show_errors,Message})->
	HideErrorsLink=#link{id=hide_errors_link, 
						 text=Message, 
						 postback={hide_errors,Message}, 
						 class="error_link"},
	wf:replace(show_errors_link,HideErrorsLink),
	wf:wire(warning_table, #appear{speed=100});

event({hide_errors,Message})->
	ShowErrorsLink=#link{id=show_errors_link, 
						 text=Message, 
						 postback={show_errors,Message}, 
						 class="error_link"},
	wf:replace(hide_errors_link,ShowErrorsLink),
	wf:wire(warning_table, #fade{speed=100}).
%%% ============================================================================
%%% Handlers for continous events	
continue({continue, Q}, timeout) ->
	query_result_handler(Q,timeout),
	event(load_list);

continue({continue, Q}, Result) ->
	query_result_handler(Q,Result),
	event(load_list).
%%% ============================================================================
%%% Hepler functions
background_update()->
	timer:sleep(1000),
	Queries=nitrogen_helper:get_running_queries(),
	Rows=case lists:foldl(queries_table_fun(), [], Queries) of
		[] -> EmptyListCell=#tablecell{body=[#p{body="Queries are not running now."}]},
			  [#tablerow{cells=[EmptyListCell]}];
		R -> R
	end,
	Table=#table{class="query_list_table", 
				 rows=Rows},
	wf:replace(running_queries_placeholder,
			   #panel{id=running_queries_placeholder,
					  body=[Table]}),
	wf:flush(),
	background_update().

queries_table_fun()->
	fun({QueryId, QueryStr}, AccIn)->
		   Cell1=#tablecell{body=[#p{body=split_label_into_rows(QueryStr)}],
							align="left",
							valign="middle",
							class=["query_list_tablecell", "white_border"]},
		   Cell2=#tablecell{body=[#link {text="X",
									class="delete_query_a", 
									title = "Abort query", 
									postback={abort_query,QueryId}}],
							align="left",
							valign="middle",
							class=["query_list_tablecell","width_15"]},
		   [#tablerow{cells=[Cell1, Cell2]}]++AccIn
	end.

query_result_handler(Q,{{result, no_result},{warning,no_warning},{error,no_error}})->
	wf:replace(executed_query_placeholder,
			   #panel{id=executed_query_placeholder,
					  body=[#p{body=nitrogen_helper:whitespaces_to_html(Q),
							   class="query_title"},
							#br{},
							#p{body="nitrogen_helper crashed",
							   class="error"},
							#br{}]}),
	wf:replace(query_result_placeholder, 
			   #panel{id=query_result_placeholder, body=[]});

query_result_handler(Q,{{result, no_result},{warning,no_warning},{error,E}})->
	wf:replace(executed_query_placeholder,
			   #panel{id=executed_query_placeholder,
					  body=[#p{body=nitrogen_helper:whitespaces_to_html(Q),
							   class="query_title"},
							#br{},
							#p{body=E,
							   class="error"},
							#br{}]}),
	wf:replace(query_result_placeholder, 
			   #panel{id=query_result_placeholder,body=[]});

query_result_handler(Q,{{result, Result},{warning,no_warning},{error,no_error}})->
	wf:replace(executed_query_placeholder,
			   #panel{id=executed_query_placeholder,
					  body=[#p{body=nitrogen_helper:whitespaces_to_html(Q),
							   class="query_title"},
							#br{}]}),
	wf:replace(query_result_placeholder,
			   #panel{id=query_result_placeholder,
					  body=[#table{id=query_result_table,
								   class="query_result_table",
								   rows=html_from_result(Result)},
							#br{}]});

query_result_handler(Q,{{result, Result},{warning,Warning},{error,no_error}})->
	wf:replace(executed_query_placeholder,
			   #panel{id=executed_query_placeholder,
					  body=[#p{body=nitrogen_helper:whitespaces_to_html(Q),
							   class="query_title"},
							%#br{},
							%#p{body=warning_handler(Warning),
							%   class="warning"},
							#br{}]}),
	wf:replace(warning_messages_placeholder,
			   #panel{id=warning_messages_placeholder,
					  body=Warning}),
	wf:replace(query_result_placeholder,
			   #panel{id=query_result_placeholder,
					  body=[#table{id=query_result_table,
								   class="query_result_table",
								   rows=html_from_result(Result)},
							#br{}]}).

warning_handler({Message, Warning}) when is_list(Warning)->
	[#link{id=show_errors_link, text=Message, postback={show_errors,Message}, class="error_link"},
	 #table{id=warning_table,
		   class="query_result_table",
		   rows=lists:foldl(fun({File, StartPos, Length,ErrorMessage},Acc)->
								Link=#link {text=File,
									   title = "Show", 
									   postback={show_selection, 
												 File, 
												 StartPos, 
												 StartPos+Length-1}},
								Cell1=#tablecell{body=[Link],
												align="left",
												valign="middle",
												class="query_result_tablecell"},
								Cell2=#tablecell{body=ErrorMessage,
												align="left",
												valign="middle",
												class="query_result_tablecell"},
								Acc++[#tablerow{cells=[Cell1,Cell2]}]
							end, [], Warning),
		 style="display:none;"}];

warning_handler(_)->"".

load_list_function()->
	Fun=fun(E)->
	CurUser=wf:user(),
	{SafeQuery={SQ,File, Pos},Q,Users}=E,
	ThrdCell=case lists:member(CurUser, Users) of
		true ->
			#tablecell{body=[#link {text="X",
									class="delete_query_a", 
									title = "Delete query", 
									postback={delete_query,SafeQuery}}],
							align="left",
							valign="middle",
							class=["query_list_tablecell","width_15"]};
		false ->#tablecell{body=[], class="width_15"}
	end,
	FstCell=#tablecell{body=[#link{body=
										split_label_into_rows(SQ),
										title = "Run query",
										postback={run_query,SafeQuery,Q}}],
							align="left",
							valign="middle",
							class=["query_list_tablecell", "white_border"]},
	{SndCell,InfoRow}=case {File, Pos} of
		{[C|_],_P} when is_integer(C) 
		  andalso is_integer(Pos)->TempIdCell=wf:temp_id(),
								   TempIdRow=wf:temp_id(),
								   TempIdLink=wf:temp_id(),
								   HelpLink=#link {id=TempIdLink,
												   text="?",
													class="delete_query_a", 
													title = "Show information", 
													postback={show_help,
															  TempIdLink, 
															  TempIdCell,
															  TempIdRow}},
									Cell=#tablecell{body=[HelpLink],
												align="left",
												valign="middle",
												class=["query_list_tablecell",
													   "width_15"]},
								   Cell2=#tablecell{id=TempIdCell,
													body=[#p{body="File: "++
															File++
															", Start Position: "
															++wf:to_list(Pos)}],
											  class=["info_tablecell", 
													 "red_border"]},
								   EmptyCell=#tablecell{body=["&nbsp;"], 
														class="width_15"},
									Row=#tablerow{id=TempIdRow,
												  cells=[Cell2,
														 EmptyCell,
														 EmptyCell],
												  style="display:none;"},
									{Cell, Row};
		{_,_}->{#tablecell{body=[],class="query_list_tablecell"},
				[]}
	end,
	
	wf:insert_bottom(
			  query_list,
			  [#tablerow{cells=[FstCell, SndCell, ThrdCell]}, InfoRow])
	end,
	Fun.

split_label_into_rows(Query) when length(Query)<?MAX_CHR_OF_LINE ->
	[Query];
split_label_into_rows(Query)->
	{List1,List2}=lists:split(?MAX_CHR_OF_LINE,Query),
	lists:flatten([List1]++["<br/>"]++split_label_into_rows(List2)).

html_from_result(Table) ->
	TableElements = to_table(Table),
	case TableElements of
		[] ->[#tablerow{cells=[#tablecell{body=[#p{body="No result"}],
										  align="left",
										  valign="middle",
										  class="query_result_tablecell"}]}];
		_ ->TableElements
	end.

to_table([]) -> [];

to_table([C={eq,_Text1,_Text2}]) ->
	[#tablerow{cells=[#tablecell{body=[#p{body=get_text(C)}],
								 align="left",
								 valign="middle",
								 class="query_result_tablecell"}]}];

to_table([{list,L}]) ->
	to_table({list,L});

to_table({list,L}) ->
	case L of
		[] ->[#tablerow{cells=[#tablecell{body=[#p{body="No result"}],
										  align="left",
										  valign="middle",
										  class="query_result_tablecell"}]}];
		_  -> [#tablerow{cells=[#tablecell{body=[to_html({list,L})],
										   align="left",
										   valign="middle",
										   class="query_result_tablecell"}]}] 
	end;

to_table([{group_by, Entity, list, List}|Tail]) ->
	to_table([{group_by, Entity},{list, List}| Tail]);

to_table([{group_by, Entity, list, Type, List}|Tail]) ->
	to_table([{group_by, Entity},{list, Type, List}| Tail]);

to_table([{group_by, PrevType, Entity, list, Type, List}|Tail]) ->
	to_table([{group_by, PrevType, Entity},{list, Type, List}| Tail]);

to_table([{group_by, PrevType, Entity, eq, Type, PropVal}|Tail]) ->
	to_table([{group_by, PrevType, Entity}, {eq, Type, PropVal}|Tail]);

to_table([{group_by, Entity, eq, Type, PropVal}|Tail]) ->
	to_table([{group_by, Entity}, {eq, Type, PropVal}|Tail]);

to_table([{group_by,Row1},Row2|Tail]) ->
	[#tablerow{cells=[#tablecell{body=[to_html(Row1)],
								 align="left",
								 valign="middle",
								 class="query_result_tablecell"},
					  #tablecell{body=[to_html(Row2)],
								 align="left",
								 valign="middle",
								 class="query_result_tablecell"}]}
	|to_table(Tail)];

to_table([{chain,L,End}|Tail]) ->
	[#tablerow{cells=[#tablecell{body=[to_html({chainList,L}),
									   to_html({chain_end,End})],
								 align="left",
								 valign="middle",
								 class="query_result_tablecell"}]}
	|to_table(Tail)];

to_table(R=[C|_]) when is_list(R) and is_integer(C)->
	[#tablerow{cells=[#tablecell{body=[#p{body=R}],
								 align="left",
								 valign="middle",
								 class="query_result_tablecell"}]}];

to_table(R) ->
    [#tablerow{cells=[#tablecell{body=[#p{body=io_lib:format("~p",[R])}],
								 align="left",
								 valign="middle",
								 class="query_result_tablecell"}]}].

to_html({list,[]}) -> [];
to_html({list,[H]}) -> to_html(H);
to_html({list,[H|T]}) -> [to_html(H),"<br/>",to_html({list,T})];

to_html({chainList,[]}) -> [];
to_html({chainList,[H]}) -> to_html({H,{comma,no}});
to_html({chainList,[H|T]}) -> [to_html({H,{comma,yes}}),to_html({chainList,T})];

to_html({chain_end,"\n"}) -> [];
to_html({chain_end,"*\n"}) -> "*";

to_html({X,{comma,IsComma}}) ->
	Separator = case IsComma of
					yes -> ", ";
					no -> ""
				end,
	case get_pos(X) of
		nopos -> #p{ body=get_text(X) ++ Separator};
		{File,StartPos,EndPos} ->
			#link {text=get_text(X)++Separator, 
				   title = "Show", 
				   postback={show_selection, File, StartPos, EndPos}}
	end;

to_html(X) ->
	case get_pos(X) of
		nopos ->#p{ body=get_text(X)};
		{File,StartPos,EndPos} ->
			#link {text=get_text(X), 
				   title = "Show", 
				   postback={show_selection, File, StartPos, EndPos}}
	end.


get_text({group_by,{_Pos,Text}}) when is_list(Text)-> Text;
get_text({group_by,{_Pos,Text}}) -> io_lib:format("~p",[Text]);

get_text({_Pos,Text}) when is_list(Text)-> Text;
get_text({_Pos,Text}) -> io_lib:format("~p",[Text]);

get_text({eq,Text1,Text2}) when is_list(Text1) and is_list(Text2)-> 
	Text1 ++ " = " ++ Text2;
get_text({eq,Text1,Text2}) ->
	io_lib:format("~p",[Text1]) ++ " = " ++ io_lib:format("~p",[Text2]);
get_text(_) -> "notext".

get_pos({group_by,{{File,StartPos,EndPos},_Text}}) ->
    get_pos({{File,StartPos,EndPos},_Text});
get_pos({{File,StartPos,EndPos},_Text}) ->
    {File, StartPos, EndPos};
get_pos(_) -> nopos.

