%%% -*- coding: latin-1 -*-

%%% The  contents of this  file are  subject to  the Erlang  Public License,
%%% Version  1.1, (the  "License");  you may  not  use this  file except  in
%%% compliance  with the License.  You should  have received  a copy  of the
%%% Erlang  Public License  along  with this  software.  If not,  it can  be
%%% retrieved at http://plc.inf.elte.hu/erlang/
%%%
%%% Software  distributed under  the License  is distributed  on an  "AS IS"
%%% basis, WITHOUT  WARRANTY OF ANY  KIND, either expressed or  implied. See
%%% the License  for the specific language governing  rights and limitations
%%% under the License.
%%%
%%% The Original Code is RefactorErl.
%%%
%%% The Initial Developer of the  Original Code is Etvs Lornd University.
%%% Portions created  by Etvs  Lornd University are  Copyright 2009,
%%% Etvs Lornd University. All Rights Reserved.

%%% @author Viktoria Fordos <v@fviktoria.hu>

-module(nitrogen_helper).
-vsn("$Rev: 6693 $ ").
-export([start_nitrogen/0, start_nitrogen/1, stop_nitrogen/0]).
-export([get_nitrogen_index/0, get_nitrogen_root/0, get_nitrogen_apps_dir/0,
         get_nitrogen_site_dir/0]).
-export([get_file_browser_server_root/0,get_file_browser_loaded_files_root/0,
         get_loaded_files_list/1]).
-export([close_qtab/0,delete_from_qtab/2,init_qtab/0,query_list_from_qtab/1]).
-export([execute_query/1,execute_system_query/2,get_running_queries/0,
         kill_query/1]).
-export([add_possible_warning/0, get_possible_warning/0]).
-export([add_to_db/1, drop_from_db/1, need_to_update/0]).
-export([do_autocomplete/1]).
-export([generate_dependency_graph/2, delete_dependency_graphs_files/1]).
-export([get_source/1]).
-export([whitespaces_to_html/1]).

-include("ui.hrl").
-include_lib("referl_core/include/core_export.hrl").

-define(TAB, query_table_v3).
-define(NROOT,"nitrogen").
-define(NSITE,?NROOT++"/site").
-define(NAPPS,?NROOT++"/apps").
-define(FB_TAB,file_browser_table).

%%% ============================================================================
%%% Start, configure and stop nitrogen framework

%% @doc Starts nitrogen framework
start_nitrogen(["from_script", YPath, YName, YPort, YListen,BrowserRoot, ImgRoot]) ->
    start_nitrogen(YPath, YName, YPort, YListen, BrowserRoot, ImgRoot);

start_nitrogen(PropList) ->
    YPort = proplists:get_value(yaws_port,PropList,"8001"),
    YListen = proplists:get_value(yaws_listen,PropList,"127.0.0.1"),
    YName = proplists:get_value(yaws_name,PropList,"localhost"),
    YPath = proplists:get_value(yaws_path,PropList,"no_path"),
    BRoot = proplists:get_value(browser_root,PropList,"no_path"),
    IRoot = proplists:get_value(images_dir,PropList,"no_path"),
    ImgRoot=case IRoot of
            "no_path" -> {ok,CwdDir}=file:get_cwd(),
                         CwdDir;
            Dir -> Dir
            end,
    start_nitrogen(YPath, YName, YPort, YListen, BRoot, ImgRoot).

start_nitrogen() ->
    start_nitrogen([]).

start_nitrogen(YPath, YName, YPort, YListen, BrowserRoot, ImgRoot)->
    try    
        load_needed(),
        init_fbtab(),
        set_file_browser_docroot(BrowserRoot),
        set_image_root(ImgRoot),
        PropList=make_proplist(YPath, YName, YPort, YListen),
        
        application:start(nprocreg),
        web_helper:start_yaws(PropList)
    catch
        RefErr ->
            io:format(?Error:error_text(RefErr) ++ "~n"),
            io:format("~p~n",[RefErr]),
            exit("Cannot start nitrogen.")
    end.

%% @doc Stop nitrogen framework
stop_nitrogen()->
    close_qtab(),
    application:stop(yaws).

%%% ============================================================================
%%% Getters for nitrogen framework

%% @doc Returns nitrogen's docroot path
get_nitrogen_index()->
    filename:join([get_base(),?NSITE,"static"]).

%% @doc Returns nitrogen's root path
get_nitrogen_root()->
    filename:join([get_base(),?NROOT]).

%% @doc Returns nitrogen's apps path
get_nitrogen_apps_dir()->
    filename:join([get_base(),?NAPPS]).

%% @doc Returns nitrogen's site path
get_nitrogen_site_dir()->
    filename:join([get_base(),?NSITE]).

%% @doc Returns the root dir(s) of browsable files in server
get_file_browser_server_root()->
    [{_,Result}]=find_in_fbtab("server_docroot"),
    Result.

%% @doc Returns the root dir(s) of browsable files
%% which previously loaded to database
get_file_browser_loaded_files_root()->
    [{_,Result}]=find_in_fbtab("db_docroot"),
    Result.

%% @doc Returns a list of filenames, where file's parent is the given argument,
%% and file's been loaded to database 
get_loaded_files_list(Parent)->
    {result,FileList}=nitrogen_helper:execute_system_query("files",
                                         {query_display_opt,
                                          [{positions, scalar}],
                                          needed_pattern,$:}),
    %Really slow if DB contains large number of files
    %FileList=case make_ui_request({filelist}) of
    %    {error, {_Code, Msg}} -> throw(?RefErr0r("Error: " ++ Msg));
    %    {ok, R} ->  R
    %end,
    lists:foldl(fun(File, List)->
                        case lists:prefix(Parent, File) of
                            true -> List ++ [File];
                            false -> List 
                        end 
                end, [], lists:map(fun(F)->lists:sublist(F,length(F)-1) end,
                                   FileList)).

%%% ============================================================================
%%% Operations on 'query_table_v3' dets table

%% @doc Returns previous queries from the given user in the query_table_v3
query_list_from_qtab(Usr) ->
    L = dets:match_object(?TAB,{'_','_','_','_','_'}),
    L2 = case Usr of
            "all" -> L;
            _ -> lists:filter(
                    fun({_SafeQuery,_Q,_R,Users,_H}) ->
                        lists:member(Usr,Users)
                    end, L)
        end,
    lists:keysort(1,lists:map(
                    fun ({SafeQuery,Q,_R,U,_H})->
                            {SafeQuery,Q,U} 
                    end, L2)).

%% @doc Deletes the given query from the given user in the query_table_v3
delete_from_qtab(SafeQuery,Usr) ->
    [{SafeQuery,Query,Result,UserList,Hash}] = find_in_qtab(SafeQuery),
    UserList2 = lists:delete(Usr,UserList),
    case UserList2 of
        [] -> delete_from_qtab(SafeQuery);
        _ -> insert_to_qtab(SafeQuery,Query,Result,UserList2,Hash)
    end.

%% @doc Initialize query_table_v3
init_qtab() ->
    dets:open_file(?TAB,[]).

%% @doc Close query_table_v3
close_qtab() ->
    dets:close(?TAB).

%%% ============================================================================
%%% Semantic Queries

%% @doc Returns the given query's result
%% and administrate the action if necessary
execute_query({_,[],_}) ->
    execute_query(noquery);

execute_query({'query',Q=[C|_],User}) when is_integer(C)->
    execute_query({'query',{Q, undefined, undefined},User});

execute_query({'query',{Query, File, Pos},User}) ->
    init_qtab(),
    SafeQuery=lists:filter(query_filter_fun(), Query),
    StartOpt=case {File, Pos} of
                 {[C|_],P} when is_integer(C) andalso is_integer(P)-> 
                     [{file, File},{position, Pos}];
                 _ ->[]
             end,
    case calculate_result(Query,[{positions,scalar}, {output,other}], StartOpt) of
        {result, Result} ->
            [R0] = ?MISC:pgetu([result],Result),
            R = io_lib:format("~p",[R0]),
            Hash = get_database_hash(),
            update_tab_if_needed({SafeQuery, File, Pos},Query,User,R,Hash),
            {{result,R0},{warning,get_possible_warning()},{error,no_error}};
        E ->
            {{result, no_result},{warning,no_warning},{error,error_handler(E)}}
    end;

execute_query({prev_query,SQ={_Query, File, Pos},U}) ->
    init_qtab(),
    [{_SQ,Q,R,_U, H}] = find_in_qtab(SQ),
    case is_database_changed(H) of
        true -> execute_query({'query',{Q, File, Pos},U});
        false ->
            update_tab_if_needed(SQ,Q,U,R,H),
            {{result,list_to_term(lists:flatten(R))},
             {warning,get_possible_warning()},
             {error,no_error}}
    end;

execute_query(noquery) ->
    [{result,[]}].

%% @doc Returns the given query's result in list
execute_system_query(Query,{query_display_opt,Option,needed_pattern,Pattern})->
    {result,R}=calculate_result(Query,Option),
    [Result] = ?MISC:pgetu([result],R),
    get_result_list(Result,Pattern).
%%% ============================================================================
%%% Running queries
get_running_queries()->
    RunningQueries=?Transform:get_running_queries(),
    [{QId,proplists:get_value(querystr,Args)} || 
     {QId, _Pid, _Mod, Args} <- RunningQueries].

kill_query(QueryId) when is_integer(QueryId)->
    ?Transform:kill_query(QueryId);
kill_query(_)->
    not_found.
%%% ============================================================================
%%% Autocomplete semantic queries

do_autocomplete(QueryString) when is_atom(QueryString) ->
    do_autocomplete(atom_to_list(QueryString));

do_autocomplete(QueryString)->
    List=refusr_ac:run(filtered_query(QueryString)),
    [{lists:flatten(InCompleteEnt++Completion), 
      lists:flatten(QueryString++Completion)}||
     {InCompleteEnt,Completion=[C|_]}<-List, is_list(InCompleteEnt),is_integer(C)].
%%% ============================================================================
%%% Helper functions for Nitrogen framework

%% @doc Turns whitespaces into HTML tags.
whitespaces_to_html(String) ->
lists:map(fun (E)->
                if 
                    E == 10 -> "<br/>";
                    E == 9  -> "&nbsp;&nbsp;";
                    E == 32 -> "&nbsp;";
                    true -> E
                end
          end, String).

%% @doc Returns file contents of the given filename
get_source(FileName)->
    case file:open(FileName,[read]) of
        {ok,_IODevice} ->
            case file:read_file(FileName) of
                {ok, Data} -> {data,Data};
                {error,Reason} -> {error, "Error: " ++ Reason}
            end;
        {error,Reason} ->
            {error, io_lib:format("Error: ~p",[Reason])}
    end.

%%% ============================================================================
%%% Database's operations

%% @doc Add the given file, or directory to database
add_to_db(File=[C|_]) when is_integer(C) ->
    case io_lib:deep_char_list(File) of
        true ->
            case make_ui_request({add_dir,File}) of
                {error, {_Code, Msg}} ->{error, "Error: " ++ Msg};
                R -> {result, R}
            end;
        false ->
            {error,"Error: bad file argument given"}
    end;
add_to_db(File)->
    {error, "Bad argument: "++File}.

%% @doc Drop the given file from database
drop_from_db(File=[C|_]) when is_integer(C) ->
    case make_ui_request({drop,File}) of
        {error, {_Code, Msg}} ->{error, "Error: " ++ Msg};
        R -> {result, R}
    end;
drop_from_db(File)->
    {error, "Bad argument: "++File}.

need_to_update() ->
    OldServerDocRoot = get_file_browser_server_root(),
    case OldServerDocRoot == get_file_browser_loaded_files_root() of
        true -> set_file_browser_docroot("no_path");
        false -> set_file_browser_docroot(OldServerDocRoot)
    end,
    true.

get_possible_warning() ->
    case is_error_in_database() of
        false -> no_warning;
        true ->   error_handler(warning)
    end.

add_possible_warning() ->
    case get_error_forms_in_database() of
        [] -> no_warning;
        Errors ->   error_handler({warning, Errors})
    end.

%%% ============================================================================
%%% Dependency graph
%% @doc Converts to SVG from the given .dot file
svg(Target) when is_atom(Target) ->
    svg(atom_to_list(Target));

svg(Target) when is_list(Target) ->
    DotName = filename:absname(Target) ++ ".dot",
    case file:read_file_info(DotName) of
        {ok, _ } -> ok;
        {error,E} -> throw(?RefError(file_not_exists,[DotName,E]))
    end,
    Res = os:cmd("dot -Tsvg "++DotName++" -o"++Target++".svg"),
    case Res of
        [] -> ok;
        _  -> {error,Res}
    end.

%% @doc Generate dependecy graph with the given options
generate_dependency_graph(Options,User) when is_atom(User)->
    generate_dependency_graph(Options,atom_to_list(User));

generate_dependency_graph(Options,User) when is_list(User)->
    {ok,CwdDir}=file:get_cwd(),
    [{_,TargetDir}]=find_in_fbtab("images_docroot"),
    DName=
    try
        case ri:draw_dep(Options) of
            {error, E} -> {error,E};
            {_ , _ } -> get_dot_name(Options);
            ok -> get_dot_name(Options)
        end
    catch
        Err={M,_,_} when is_atom(M) ->
            {error, ?Error:error_text(Err)};
        _ -> {error, unknown}
    end,
    try
        DotName=case DName of
            {error, _ } -> DName;
            {Candidate1,Candidate2} ->
                case file:read_file_info(
                        filename:absname(Candidate1) ++ ".dot") of
                    {ok, _ } -> Candidate1;
                    {error, _ } -> Candidate2
                end;
            _ ->DName
        end,
        case DotName of
            {error, _ } -> {DotName, DotName};
            _ ->
                case svg(DotName) of
                    ok -> 
                        file:copy(filename:join(CwdDir,DotName++".dot"),
                                  filename:join([TargetDir,
                                                User++"_"++DotName++".dot"])),
                        file:delete(filename:join(CwdDir,DotName++".dot")),
                        file:copy(filename:join(CwdDir,DotName++".svg"),
                                  filename:join([TargetDir,
                                                User++"_"++DotName++".svg"])),
                        file:delete(filename:join(CwdDir,DotName++".svg")),
                        {filename:join([TargetDir,User++"_"++DotName++".dot"]),
                         filename:join([TargetDir,User++"_"++DotName++".svg"])};
                    Error  ->
                        file:copy(filename:join(CwdDir,DotName++".dot"),
                                  filename:join([TargetDir,
                                                User++"_"++DotName++".dot"])),
                        file:delete(filename:join(CwdDir,DotName++".dot")), 
                        {filename:join([TargetDir,User++"_"++DotName++".dot"]),Error}
                end
        end
    catch
        Err_={M_,_,_} when is_atom(M_) ->
            {error, ?Error:error_text(Err_)};
        _ ->{error, unknown}
    end;

generate_dependency_graph(_Options,_User) ->
    {error, "Bad argument was given"}.

%% @doc Deletes all of the files which are associated to the user
delete_dependency_graphs_files(User) when is_atom(User)->
    delete_dependency_graphs_files(atom_to_list(User));

delete_dependency_graphs_files(User) when is_list(User)->
    [{_,TargetDir}]=find_in_fbtab("images_docroot"),
    case file:list_dir(TargetDir) of
        {ok,FileNames}->
            lists:foreach(fun(Elem) ->
                                 case lists:prefix(User++"_",Elem) 
                                 andalso (lists:suffix(".dot",Elem) 
                                 orelse lists:suffix(".svg",Elem)) of
                                    true -> 
                                        file:delete(
                                            filename:join([TargetDir, Elem]));
                                    false -> ok
                                 end
                          end, FileNames);
        _ -> ok
    end.
    
%%% ============================================================================
%%% helper functions
get_dot_name(Options)->
    case Options of
        [{level, func}, {type,cycles}] -> {"dep_func_cycles","dep_func"};
        [{level, func}, {type, _}] -> "dep_func";
        [{level, mod}, {type,cycles}] -> {"dep_mod_cycles","dep_mod"};
        [{level, mod}, {type, _}]  -> "dep_mod";
        [{level, mod}, {gnode, Module}] when is_atom(Module) and 
                                                 (Module /= undefined)->
            "dep_mod_"++atom_to_list(Module);
        [{level, func}, {gnode, Function=[C|_]}] when is_list(Function) and 
                                                          is_integer(C)->
            "dep_func_"++lists:flatten(lists:map(fun(E)->
                                                        case E of
                                                            58 ->"_";
                                                            47 ->"_";
                                                            _  ->E
                                                        end
                                                 end,Function));
        _ -> {error, "not_recognised_option"}
    end.

get_result_list(QueryResult,NeededPattern)->
    ResultList=string:tokens(QueryResult," \t\n"),
    case NeededPattern of
        none -> {result,ResultList};
        _      -> {result, lists:filter(fun(E)->
                                             case string:chr(E,NeededPattern) of
                                                 0 -> false;
                                                 _ -> true 
                                             end 
                                      end,ResultList)}
    end.

set_image_root(ImgRoot=[C|_]) when is_integer(C) ->
    insert_to_fbtab("images_docroot",ImgRoot);

set_image_root(ImgRoot) when is_atom(ImgRoot) ->
    set_image_root(atom_to_list(ImgRoot)).

set_file_browser_docroot(no_path) ->
    set_file_browser_docroot("no_path");

set_file_browser_docroot("no_path")->
    Root=calculate_file_browser_serverroot(),
    set_file_browser_docroot(Root, no_path);

set_file_browser_docroot(BrowserRoot) when is_atom(BrowserRoot) ->
    set_file_browser_docroot([atom_to_list(BrowserRoot)], path);

set_file_browser_docroot(BrowserRoot=[C|_]) when is_integer(C) ->
    set_file_browser_docroot([BrowserRoot],path);

set_file_browser_docroot(BrowserRoot=[C|_]) when is_list(C) ->
    set_file_browser_docroot(BrowserRoot,path);

set_file_browser_docroot(_)->
    throw(?RefErr0r(
          "Bad argument given at startup~n Bad parameter: broswer root~n")).

set_file_browser_docroot(BrowserRoot=[C|_], path) when is_integer(C)->
    set_file_browser_docroot([BrowserRoot], path);

set_file_browser_docroot(BrowserRoot=[C|_], path) when is_list(C)->
    insert_to_fbtab("server_docroot",BrowserRoot),
    insert_to_fbtab("db_docroot",calculate_file_browser_dbroot());

set_file_browser_docroot(_, path)  ->
    throw(?RefErr0r(
          "Bad argument given at startup~n Bad parameter: broswer root~n"));

set_file_browser_docroot(BrowserRoot, no_path)->
    insert_to_fbtab("server_docroot",BrowserRoot),
    insert_to_fbtab("db_docroot",BrowserRoot).

calculate_file_browser_serverroot()->
    calculate_file_browser_dbroot().

calculate_file_browser_dbroot()->
    %Gets filelist from UI router
    {result, FileList}=nitrogen_helper:execute_system_query("files",
                                         {query_display_opt,
                                          [{positions, scalar}],
                                          needed_pattern,$:}),
    %Gets top level directories
    RootDir=calculate_root(lists:map(fun(File)->
                                            lists:sublist(File,length(File)-1) 
                                    end,FileList)),
    
    case RootDir of
        [] -> case file:get_cwd() of
                  {ok,Dir} -> [Dir];
                  {error, Error}->
                      throw(?RefErr0r("Error during init: " ++ 
                                          io_lib:format("~p", [Error])))
              end;
        _  -> RootDir
    end.

calculate_root(FileList=[C|_]) when is_list(C)->
    lists:usort(lists:map(fun(E) -> filename:dirname(E) end, FileList));

calculate_root([])->[];

calculate_root(FileList)->
    throw(?RefError(bad_argument,[FileList])).

make_ui_request(Args)->
    %make request to UI router
    ReqID = ?UI:getid(),
    ok = ?UI:request(ReqID,Args),
    receive
        {ReqID, reply, R} -> R
    end.

get_base()->
    Path0 = filename:split(filename:dirname(code:which(nitrogen_helper))),
    Path =filename:join(lists:sublist(Path0,length(Path0)-1)),
    Path.

%% @doc Return the filter function,
%% which need to remove whitespaces from queries.
query_filter_fun()->
fun(E)->
        if 
            E == 10 -> false;
            E == 9  -> false;
            E == 32 -> false;
            true -> true
        end
end.

%% @doc Replaces group of whitespace in the given query with 1 blank char.
filtered_query(Query=[C|_]) when is_integer(C) ->
    case regexp:gsub(Query, "(\s|\t|\n)+"," ") of
        {ok,NewString,_RepCount} -> NewString;
        {error, _Error} -> Query
    end.

load_needed() ->
    BasePath=get_base(),
    %Needed to start Nitrogen
    case code:add_path(filename:join([BasePath,?NAPPS,"nprocreg/ebin"])) of        
        true -> ok;
        {error,_} -> throw(?RefError(file_notdir,
                                     [filename:join([BasePath,
                                                     ?NAPPS,
                                                     "nprocreg/ebin"])]))
    end,
    case code:add_path(filename:join([BasePath,?NAPPS,"simple_bridge/ebin"])) of
        true -> ok;
        {error,_} -> throw(?RefError(file_notdir,
                                     [filename:join([BasePath,
                                                    ?NAPPS,
                                                    "simple_bridge/ebin"])]))
    end,
    case code:add_path(filename:join([BasePath,?NAPPS,"nitrogen/ebin"])) of
        true -> ok;
        {error,_} -> throw(?RefError(file_notdir,
                                    [filename:join([BasePath,
                                                    ?NAPPS,
                                                    "nitrogen/ebin"])]))
    end,
    case code:add_path(filename:join([BasePath,?NAPPS,"simple_bridge/include"])) of
        true -> ok;
        {error,_} -> throw(?RefError(file_notdir,
                                     [filename:join([BasePath,
                                                    ?NAPPS,
                                                    "simple_bridge/include"])]))
    end,
    case code:add_path(filename:join([BasePath,?NAPPS,"nitrogen/include"])) of
        true -> ok;
        {error,_} -> throw(?RefError(file_notdir,
                                    [filename:join([BasePath,
                                                    ?NAPPS,
                                                    "nitrogen/include"])]))
    end,
    case code:ensure_loaded(nprocreg) of
        {error,_} -> throw(?RefErr0r(nprocreg_not_loaded));
        _ -> ok
    end,
    
    %Nitrogen framework use this low-level module to load 
    %the requested page's main modul in some circumstances, 
    %so we should add the page's ebin folder
    {ok,LoadPath}=erl_prim_loader:get_path(),
    erl_prim_loader:set_path(LoadPath++[filename:join([BasePath,?NSITE,"ebin"])]),
    ok.

make_proplist(YPath, YName, YPort, YListen) ->
    try
    YawsPathProp = {yaws_path,YPath},
    YawsNameProp = {yaws_name,YName},
    YawsPortProp = {yaws_port, web_helper:convert_list_to_integer(YPort)},
    YawsListenProp = {yaws_listen, web_helper:string_to_ip(YListen)},
    YawsIndexFunModuleProp = {yaws_index_fun_module, nitrogen_helper},
    YawsIndexFunNameProp = {yaws_index_fun_name, get_nitrogen_index},
    YawsPostSize = {yaws_partial_post_size, 1024*1024},
    YawsAppMod = {yaws_app_mod,   [{"/", nitrogen_yaws}] },
    NitrogenProp = {nitrogen_prop,with_nitrogen},
    AllProp = [YawsNameProp,YawsPortProp,YawsListenProp,YawsIndexFunModuleProp,
               YawsIndexFunNameProp,YawsAppMod,NitrogenProp,YawsPostSize],
    case YPath of
        "no_path" -> AllProp;
        _ -> [YawsPathProp|AllProp]
    end
    catch
        RefErr ->
            io:format(?Error:error_text(RefErr) ++ "~n"),
            exit("Cannot start nitrogen.")
    end.


%% @doc Administrate query_table_v3: if User has not owned the query,
%% add User to owner list.
update_tab_if_needed(SafeQuery,Query,User,Result,Hash)->
    Users = case find_in_qtab(SafeQuery) of
                [] -> [User];
                [{_SQ,_Q,_R,UL,_H}] ->case lists:member(User,UL) of
                                          true -> UL;
                                          false -> [User|UL]
                                      end
    end,
    insert_to_qtab(SafeQuery,Query,Result,Users,Hash).

%% @doc Returns with the result of the given query
calculate_result(Q,DisplayOpt)->
    calculate_result(Q,DisplayOpt, []).

calculate_result(Q,DisplayOpt, StartOpt) when is_list(StartOpt)->
    Req = {transform,semantic_query,
           [{ask_missing,false},
            {querystr,Q},
            {display_opt,DisplayOpt},
            {start_opt,StartOpt}]},
    case make_ui_request(Req) of
        {ok, Result} -> Result;
        M -> M
    end;

calculate_result(Q,DisplayOpt, _)->
    calculate_result(Q,DisplayOpt, []).

get_database_hash() ->
    HashList = lists:filter(fun(X) -> X /= virtual end,
                            [(refcore_graph:data(Node))#form.hash
                            || Node <- refcore_graph:path(
                                 refcore_graph:root(), [file, form])]),
    erlang:phash2(HashList).

error_handler(warning) ->
    "Warning: the database contains file(s) with error(s).";

error_handler({warning, Errors}) ->
    {"Warning: the database contains file(s) with error(s).", Errors};

error_handler(E) ->
    case E of
        {abort, M} -> reflib_error:error_text(M);
        {error, M} -> "Fatal error: " ++ io_lib:format("~p",[M])
    end.

is_error_in_database() ->
    case ?Query:exec(?Query:seq([file],?File:error_forms())) of
        [] -> false;
        _ -> true
    end.

get_error_forms_in_database() ->
    case ?Query:exec(?Query:seq([file],?File:error_forms())) of
        [] -> [];
        ErrorForms -> get_errors(ErrorForms)
    end.

get_errors([])->[];

get_errors(Forms) when is_list(Forms)->
    ErrorFiles=?Query:exec(Forms,[{form, back}]),
    lists:foldl(fun({File, Form}, Acc)->
                       {Index, First}=case ?Syn:index(File, form, Form) of
                                 1 -> {1,true};
                                 Int -> {Int-1,false}
                             end,
                       {StartPos,Length} = case First of
                            true->[Token]=?Query:exec(Form, [{flex, 1}]),
                                     case ?Token:pos(Token) of
                                         {SPos,_} ->Len=(?ESG:data(Form))#form.length,
                                                    {SPos,Len};
                                         not_found -> {not_found, 0}
                                     end;
                            false->PrevForm=?Query:exec(File, ?File:form(Index)),
                                      [Token]=?Query:exec(PrevForm, [{flex, last}]),
                                      case ?Token:pos(Token) of
                                          {SPos, _} ->Len=(?ESG:data(Form))#form.length
                                                          +?Token:len(?Token:data(Token)),
                                                      {SPos,Len};
                                          not_found -> {not_found, 0}
                                      end
                            end,
                       Tag=(?Graph:data(Form))#form.tag,
                        case {ErrorMessage=make_error_message(Tag), StartPos} of
                            {"", _} -> Acc;
                            {_, not_found} -> Acc;
                            _ -> Acc++[{?File:path(File), 
                                        StartPos, 
                                        Length,
                                        ErrorMessage}]
                        end
                end, [], lists:zip(ErrorFiles, Forms));

get_errors(_)->[].

make_error_message(Tag)->
    case Tag of
        {1,Mess} -> Mess;
        {no_include_file, Name} -> io_lib:format("Include file not found: ~s",
                                                 [Name]);
        {include_lib, application_not_specified, Name} ->
            io_lib:format("Application not specified -include lib: ~s", [Name]);
        {unknown_env_in_include, [Name]}->io_lib:format("Unknown env in include: ~s",
                                                        [Name]);
        {include_error, IncName, Reason} ->
            io_lib:format("Include error in ~s. Reason: ~s",[IncName,Reason]);
        _ -> ""
    end.

is_database_changed(Hash1) ->
    Hash1 /= get_database_hash().

list_to_term(String) ->
    {ok, T, _} = erl_scan:string(String++"."),
    case erl_parse:parse_term(T) of
        {ok, Term} -> Term;
        {error, Error} -> Error
    end.

%%% ============================================================================
%%% Low- level operations on 'file_browser_table' dets table

insert_to_fbtab(Key,Value) ->
    dets:open_file(?FB_TAB,[]),
    dets:insert(?FB_TAB,{Key,Value}),
    dets:close(?FB_TAB).

find_in_fbtab(Key) ->
    dets:open_file(?FB_TAB,[]),
    Result=dets:match_object(?FB_TAB,{Key,'_'}),
    dets:close(?FB_TAB),
    Result.

init_fbtab()->
    dets:open_file(?FB_TAB,[]),
    dets:delete_all_objects(?FB_TAB),
    dets:close(?FB_TAB).

%%% ============================================================================
%%% Low- level operations on 'query_table_v3' dets table
insert_to_qtab(SafeQuery=[C|_],Query, Result, User, Hash) when is_integer(C)->
    insert_to_qtab({SafeQuery, undefined, undefined}, Query, Result, User, Hash);

insert_to_qtab(SafeQuery={_Query, _File, _Pos},Query, Result, User, Hash) ->
    dets:insert(?TAB,{SafeQuery,Query,Result,User,Hash}).

find_in_qtab(SafeQuery) ->
    dets:lookup(?TAB,SafeQuery).

delete_from_qtab(SafeQuery) ->
    dets:delete(?TAB,SafeQuery).

